@@ -1,5 +1,5 @@
 from .bases import Pys
-from .utils import get_token_name_by_token_type
+from .constants import TOKENS
 
 class PysToken(Pys):
 
@@ -9,10 +9,14 @@ class PysToken(Pys):
         self.value = value
 
     def __repr__(self):
-        return 'Token({}{})'.format(
-            get_token_name_by_token_type(self.type),
-            '' if self.value is None else ', {!r}'.format(self.value)
-        )
+        name = '<UNKNOWN>'
+
+        for token_name, token_type in TOKENS.items():
+            if token_type == self.type:
+                name = token_name
+                break
+
+        return 'Token({}{})'.format(name, '' if self.value is None else ', value={!r}'.format(self.value))
 
     def match(self, type, value):
         return self.type == type and self.value == value
